# _*_ coding: utf-8 _*_
#
# hspylib v0.11.100
#
# Package: main.hspylib.modules.cli.tui.extra
"""Package initialization."""

__all__ = [
  'mchoose', 
  'mdashboard', 
  'minput', 
  'mselect'
]
__version__ = '0.11.100'
