# _*_ coding: utf-8 _*_
#
# hspylib v0.11.100
#
# Package: main.hspylib.core.crud.db.firebase
"""Package initialization."""

__all__ = [
  'firebase_config', 
  'firebase_repository'
]
__version__ = '0.11.100'
