# _*_ coding: utf-8 _*_
#
# hspylib v0.11.100
#
# Package: main.hspylib
"""Package initialization."""

__all__ = [
  '__classpath__', 
  '__main__', 
  'addons', 
  'core', 
  'modules'
]
__version__ = '0.11.100'
