# _*_ coding: utf-8 _*_
#
# hspylib v0.11.100
#
# Package: main.hspylib.modules.cli.vt100
"""Package initialization."""

__all__ = [
  'terminal', 
  'vt_100', 
  'vt_codes', 
  'vt_colors', 
  'vt_utils'
]
__version__ = '0.11.100'
